@@ -1,3 +1,4 @@
+# pylint: disable=too-many-arguments,invalid-name
 # !/usr/bin/env python
 #
 # Copyright 2014 Google Inc. All Rights Reserved.
@@ -53,8 +54,8 @@ parser.add_argument(
     "refresh token. Each scope should be separated by a comma.",
 )
 
-
-class ClientConfigBuilder(object):
+# pylint disable=too-few-public-methods
+class ClientConfigBuilder:
     """Helper class used to build a client config dict used in the OAuth 2.0 flow."""
 
     _DEFAULT_AUTH_URI = "https://accounts.google.com/o/oauth2/auth"
@@ -119,7 +120,7 @@ def main(client_id, client_secret, scopes):
 
     print(
         "Log into the Google Account you use to access your AdWords account "
-        "and go to the following URL: \n%s\n" % auth_url
+        f"and go to the following URL: \n{auth_url}\n"
     )
     print("After approving the token enter the verification code (if specified).")
     code = input("Code: ").strip()
@@ -127,8 +128,8 @@ def main(client_id, client_secret, scopes):
     try:
         flow.fetch_token(code=code)
     except InvalidGrantError as ex:
-        print("Authentication has failed: %s" % ex)
+        print(f"Authentication has failed: {ex}")
         sys.exit(1)
 
-    print("Access token: %s" % flow.credentials.token)
-    print("Refresh token: %s" % flow.credentials.refresh_token)
+    print(f"Access token: {flow.credentials.token}")
+    print(f"Refresh token: {flow.credentials.refresh_token}")
