@@ -1,3 +1,4 @@
+# pylint: disable=no-member,too-many-locals
 """
     CUSTOM READERS CLASSES
         - Class which manages reader tasks like auth, requests, pagination
@@ -6,7 +7,7 @@ import datetime
 import os
 import socket
 import sys
-from typing import Callable
+from typing import Callable, List, Dict
 
 import httplib2
 from googleapiclient.discovery import build
@@ -20,6 +21,9 @@ from sdc_dp_helpers.api_utilities.tracking_metadata import tracking_metadata
 
 
 class GAV3Reader:
+    """
+    Google analytics v3
+    """
     def __init__(
         self, service_account_email, creds_filepath, config_filepath, **kwargs
     ):
@@ -147,18 +151,18 @@ class GAV3Reader:
                     if not response:
                         print(f"No response for view_id: {view_id}")
                         break
-                    elif len(response.get("rows", [])) < 1:
+                    if len(response.get("rows", [])) < 1:
                         print("No more data in response.")
                         break
                     page_token += response.get("itemsPerPage", 0)
 
-                    date_dataset.extend( self._normalize_data(response) )
+                    date_dataset.extend(self._normalize_data(response))
 
                 # generator preserves memory, additional data provided
                 # to efficiently partition to s3
                 # only yield if data is returned.
                 if date_dataset:
-                    # TODO: use self.tracking_object byte_size attrib 
+                    # TO DO: use self.tracking_object byte_size attrib
                     #       to determine whether or not to yield here
                     yield {"data": date_dataset, "date": date, "view_id": view_id}
                 else:
@@ -181,6 +185,33 @@ class BaseGAV4Reader:
     analytics = None
     max_pages = None
 
+    @staticmethod
+    def _normalize_gav4(response: List[Dict], view_id: str) -> list:
+        for idx, report in enumerate(response):
+            if idx < len(report):
+                column_header = report.get("columnHeader", {})
+                dimension_headers = column_header.get("dimensions", [])
+                metric_headers = column_header.get("metricHeader", {}).get(
+                    "metricHeaderEntries", []
+                )
+                rows = report.get("data", {}).get("rows", [])
+                dataset = []
+                for row in rows:
+                    row_data = {}
+                    dimensions = row.get("dimensions", [])
+                    date_range_values = row.get("metrics", [])
+
+                    for header, dimension in zip(dimension_headers, dimensions):
+                        row_data[header] = dimension
+                    for  values in enumerate(date_range_values):
+                        for metric_header, value in zip(
+                            metric_headers, values.get("values")
+                        ):
+                            row_data[metric_header.get("name")] = value
+                        row_data["view_id"] = view_id
+                    dataset.append(row_data)
+        return dataset
+
     class ReaderIter:
         """Inner Class - Iterator that yields queries until max pages or pageToken == None."""
 
@@ -206,21 +237,21 @@ class BaseGAV4Reader:
             # check StopIteration criteria
             if self.max_pages is not None:
                 if self.page_counter >= self.max_pages:
-                    print("stopping iteration on max pages = {}".format(self.max_pages))
+                    print(f"stopping iteration on max pages = {self.max_pages}")
                     raise StopIteration
 
             if self.page_counter > 0 and self.page_token is None:
-                print("stopping iteration on pageToken = {}".format(self.page_token))
+                print(f"stopping iteration on pageToken = {self.page_token}")
                 raise StopIteration
 
             # run get_report
             report, self.page_token = self.query_fn(
                 queries=self.query, page_token=self.page_token
             )
+            page=self.page_counter + 1
+            view_id=self.query.get("viewId")
             print(
-                "Finished query function for page {page} of view ID {view_id}".format(
-                    page=self.page_counter + 1, view_id=self.query[0].get("viewId")
-                )
+                f"Finished query function for page {page} of view ID {view_id}"
             )
 
             # increment pagination counter
@@ -250,6 +281,12 @@ class BaseGAV4Reader:
         backoff_factor=float(os.environ.get("REQUEST_BACKOFF_FACTOR", 0.01)),
         backoff_method=os.environ.get("REQUEST_BACKOFF_METHOD", 0.01),
     )
+    @retry_handler(
+        exceptions=(socket.timeout, HttpError),
+        total_tries=5,
+        initial_wait=60,
+        backoff_factor=2,
+    )
     def get_report(self, queries: list, page_token: str = None):
         """
         Use the Analytics Service Object to query the Analytics Reporting API V4.
@@ -277,17 +314,53 @@ class BaseGAV4Reader:
 
         return report, page_token
 
-    def run_query(self, query: list, page_token: str = None):
-        """
-        returns a sync iterator
-        """
+    def _query_handler(self, query: dict, page_token: str = None):
+        date_dataset = []
+        view_id = query.get("view_id")
+
+        # """
+        # returns a sync iterator
+        # """
         iter_ = self.ReaderIter(
             query_fn=self.get_report,
             query=query,
             max_pages=self.max_pages,
             page_token=page_token,
         )
-        return iter_
+        for response in enumerate(iter_):
+            if not response:
+                print(f"No response for view_id: {view_id}")
+                break
+            for values in enumerate(response):
+                date_dataset.extend(self._normalize_gav4(response, view_id))
+                print(f"length of dataset is {len(date_dataset)}")
+                token = values.get("nextPageToken")
+                if not token:
+                    break
+                page_token = token
+                print(f"page token is {page_token}")
+
+        return date_dataset
+
+    def run_query(self):
+        date_dataset: list = []
+        for view_id in self.config.get("view_ids"):
+            for date in date_range(
+                start_date=self.config.get("start_date"),
+                end_date=self.config.get("end_date"),
+            ):
+                print(f"Gathering data for {view_id} at date: {date}.")
+                query: dict = self.build_query(self.config, date, view_id)
+                date_dataset: list[Dict] = self._query_handler(query, page_token="1")
+                # generator preserves memory, additional data provided
+                # to efficiently partition to s3
+                # only yield if data is returned.
+                if date_dataset:
+                    # TO DO: use self.tracking_object byte_size attrib
+                    #       to determine whether or not to yield here
+                    yield {"data": date_dataset, "date": date, "view_id": view_id}
+                else:
+                    print("View ID had no data for given date, skipping.")
 
 
 class CustomGoogleAnalyticsReaderWithServiceAcc(BaseGAV4Reader):
@@ -309,6 +382,7 @@ class CustomGoogleAnalyticsReaderWithServiceAcc(BaseGAV4Reader):
         )
         # build analytics object
         self.analytics = self.build_analytics_service_object()
+        self.config = load_file(kwargs.get("config_filepath"), "yaml")
 
     @staticmethod
     def auth_credentials(service_account_secrets_path, service_account_email, scopes):
@@ -327,3 +401,16 @@ class CustomGoogleAnalyticsReaderWithServiceAcc(BaseGAV4Reader):
         return ServiceAccountCredentials.from_p12_keyfile(
             service_account_email, service_account_secrets_path, scopes=scopes
         )
+
+    @staticmethod
+    def build_query(config: dict, date: str, view_id: str) -> List[Dict]:
+        query = {
+            "dateRanges": {"startDate": date, "endDate": date},
+            "metrics": [{"expression": metric} for metric in config.get("metrics", [])],
+            "dimensions": [
+                {"name": dimension} for dimension in config.get("dimensions", [])
+            ],
+            "viewId": view_id,
+        }
+
+        return [query]
